from distutils.core import setup

setup(
    name='py3-base92',
    version='1.1.0',
    author='Gu-f',
    packages=['src/py3base92'],
    scripts=[],
    url='https://github.com/Gu-f/py3base92',
    license='LICENSE',
    description='A python library to create base92 encoded strings in python3.',
    long_description=open('README.md', encoding="utf-8").read(),
    install_requires=[],
)
